/**
 * Author: Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */
package org.objectweb.fractal.bf.connectors.rmi;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.rmi.Remote;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

/**
 * 
 */
public class RmiInvocationHandler implements InvocationHandler {

	/**
	 * the name of the service this invocation handler will dispatch calls to.
	 */
	private final String name;

	/**
	 * @param params
	 */
	public RmiInvocationHandler(String name) {
		this.name = name;
	}

	/**
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object,
	 *      java.lang.reflect.Method, java.lang.Object[])
	 */
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {
		Registry registry = LocateRegistry
				.getRegistry(RmiConnectorConstants.JAVA_RMI_REGISTRY_DEFAULT_PORT);

		Remote remote = registry.lookup(this.name);

		final Method remoteMethod = remote.getClass().getMethod(
				method.getName(), method.getParameterTypes());

		Object result = remoteMethod.invoke(remote, args);

		return result;
	}
}
